{# def
label,
name,
id,
value,
checked=false,
disabled=false,
onclick="",
#}
<div class="{{attrs.div_class or RADIO_DIV_CLASS}}">
  <input type="radio" name="{{name}}" id="{{id}}" value="{{value}}" {% if checked %}checked{% endif %}
    class="{{attrs.class or RADIO_INPUT_CLASS}}" {% if onclick %}onclick="{{onclick}}" {% endif %} {%if
    disabled%}disabled{%endif%}>
  <Label :for="id" :class="attrs.label_class or RADIO_LABEL_CLASS">{{label}}</Label>
</div>
