{# def id=None, mode: Literal["micro","mini","outline","solid"] = "solid"  #}

{% if mode == "micro"%}<icons.micro.ListBullet :id="id" :class="attrs.class or ''"/>
{% elif mode == "mini"%}<icons.mini.ListBullet :id="id" :class="attrs.class or ''"/>
{% elif mode == "outline"%}<icons.outline.ListBullet :id="id" :class="attrs.class or ''"/>
{% elif mode == "solid"%}<icons.solid.ListBullet :id="id" :class="attrs.class or ''"/>
{%endif%}
