{# def id=None, mode: Literal["mini","outline","solid"] = "solid"  #}

{% if mode == "mini"%}<icons.mini.ArrowSmallLeft :id="id" :class="attrs.class or ''"/>
{% elif mode == "outline"%}<icons.outline.ArrowSmallLeft :id="id" :class="attrs.class or ''"/>
{% elif mode == "solid"%}<icons.solid.ArrowSmallLeft :id="id" :class="attrs.class or ''"/>
{%endif%}
