{# def widget, children_widget #}

<pydantic_form.Widget :widget="widget" :removable="widget.removable">
  <div id="{{widget.id}}" class="m-4">
    <Details>
      <Summary :id="widget.id + '-summary'">
        <H3 :class="H3_SUMMARY_CLASS">{{widget.title}}</H3>
        <pydantic_form.Error :text="widget.error" />
      </Summary>
      <div>
        {% for child in children_widget %}
        {{ child }}
        {% endfor %}
      </div>
    </Details>
  </div>
</pydantic_form.Widget>
