{# def
name,
id,
value=None,
checked=False,
#}
<input name="{{name}}" {% if value is not none %}value="{{value}}" {% endif %} {% if checked %}checked{% endif %}
  type="checkbox" id="{{id}}" class="{{attrs.class or CHECKBOX_CLASS}}" />
