{# def
method="",
action=None,
hx_post=None,
#}
<form class="{{attrs.class or FORM_CLASS}}" {% if hx_post is not none
  %}hx-post="{% if hx_post is not true %}{{hx_post}}{% endif%}" {% endif %}{% if action is not none
  %}action="{{action}}" {% endif %}{% if method %}method="{{method}}" {% endif %}>
  <CsrfToken />
  {{ content }}
</form>
